require 'account_controller'
require 'json'
require 'jwt'

class RedmineOauthController < AccountController
  def oauth
    if Setting.plugin_redmine_omniauth_client['oauth_authentification']
      session[:back_url] = params[:back_url]
      redirect_to oauth_client.auth_code.authorize_url(:redirect_uri => oauth_callback_url)
    else
      password_authentication
    end
  end

  def oauth_callback
    if params[:error]
      flash[:error] = l(:notice_access_denied, :app => settings['app_name'])
      redirect_to signin_path
    else
      begin
        token = oauth_client.auth_code.get_token(params[:code],:mode => :body, :redirect_uri => oauth_callback_url)
        info = JWT.decode(token.token, nil, false)[0]
      rescue
        flash[:error] = (t 'notices.unable_to_authenticate')
      end

      #result = token.get(settings['site_url'] + settings['ws_url'])
      #info = JSON.parse(result.body)

      if info && info[settings['field_username']]
        try_to_login info
      else
        flash[:error] = l(:notice_unable_to_obtain_app_credentials, :app => settings['app_name'])
        redirect_to signin_path
      end
    end
  end

  def try_to_login(info)
   params[:back_url] = session[:back_url]
   session.delete(:back_url)
   begin
     user = User.where(sso_u_id: info[settings['sso_u_id']]).first
     #File.open("ssouid_user.txt", 'w') { |file| file.write(user) }
   rescue
   end
   if user.nil?
      user = User.find_by_login(info[settings['field_username']])
      #File.open("username_user.txt", 'w') { |file| file.write(user) }
      #File.open("sso.txt", 'w') { |file| file.write(info[settings['sso_u_id']]) }

      #user.sso_u_id = info[settings['sso_u_id']]
      #user.save
      if user.nil?
        # Create on the fly
        user = User.new
        user.sso_u_id = info[settings['sso_u_id']]
        user.firstname = info[settings['field_firstname']]
        user.lastname = info[settings['field_lastname']]
	user.mail =  info[settings['field_email']]
	if info.key?('email_contact')
		if info['email_contact'] != ""
			user.mail = info['email_contact']
		end
	end
        user.login = info[settings['field_username']]
        user.random_password
        user.register

        # Use registration if defined
        case Setting.self_registration
        when '1'
          register_by_email_activation(user) do
            onthefly_creation_failed(user)
          end
        when '2'
          register_manually_by_administrator(user) do
            onthefly_creation_failed(user)
          end
        when '3'
          register_automatically(user) do
            onthefly_creation_failed(user)
          end
        else
          if settings['force_account_creation']
            register_automatically(user) do
              onthefly_creation_failed(user)
            end
          else
            flash[:error] = l(:unable_create_account)
            redirect_to signin_path
          end
        end
      else
        user.sso_u_id = info[settings['sso_u_id']]
        user.save
      end
   else
     user.firstname = info[settings['field_firstname']]
     user.lastname = info[settings['field_lastname']]
     user.mail = info[settings['field_email']]
     if info.key?('email_contact')
    	if info['email_contact'] != ""
		user.mail = info['email_contact']
	end
     end
     user.login = info[settings['field_username']]
     user.save

      # Existing record
      if user.active?
        successful_authentication(user)
      else
        # Redmine 2.4 adds an argument to account_pending
        if Redmine::VERSION::MAJOR > 2 or
          (Redmine::VERSION::MAJOR == 2 and Redmine::VERSION::MINOR >= 4)
          account_pending(user)
        else
          account_pending
        end
      end
    end
#########################
    #File.open("ssouidUSER.txt", 'w') { |file| file.write(user.sso_u_id) }
    begin
      # Odstraní uživatele ze všech skupin, případně jen ze skupin párovaných k SSO.
      if @settings['remove_from_groups']
        if @settings['remove_from_all_groups']
          user.groups.delete(Group.all)
        # Nebo pouze ze spárovaných skupin
        else
          GroupMapping.all.each do |group_mapping|
            group = Group.find(group_mapping.group_id)
            user.groups.delete(group) if user.groups.include? group
          end
        end
      end

      group_prefix = "f~"
      groups = info['groups']

      unless groups == nil || groups == []
        # Prochází každou skupinou v SSO acces tokenu
        groups.each do |sso_group|
          # možná mají role nějaký prefix
          sso_group = group_prefix + sso_group
          # Vyhledá mapování skupin na skupiny RM
          mappings = GroupMapping.where(variable: sso_group)
          # Pokud najde, přiřadí uživatele do příslušných skupin
          unless mappings.to_a == nil || mappings.to_a == []
            mappings.each do |mapping|
              @group = Group.find(mapping.group_id)
              unless @group.users.include? user
                @group.users << user
              end
            end
          # Pokud nenajde, hledá skupinu přímo odpovídající jménem skupině v SSO
          else
            @group = Group.find_by(lastname: sso_group)
            # Pokud nenajde, vytvoří skupinu
            if @group == nil || @group == []
              @group = Group.new({:lastname => sso_group})
              #@group.name= sso_group
              @group.save
            end
            # Vytváří mapování na novou, či nemapovanou skupinu a přiřadí uživatele do skupiny
            #File.open("mapping_" + sso_group + ".txt", 'w') { |file| file.write(@group) }
            unless @group.users.include? user
              @group.users << user
            end
            new_mapping = GroupMapping.new({:variable => sso_group, :group_id => @group.get_id})
            new_mapping.save
          end
        end
      end       # groups
   rescue
      logger.error  "Error. Group assignemnt failed."
   end
  end

  def oauth_client
    @client ||= OAuth2::Client.new(settings['client_id'], settings['client_secret'],
      :site => settings['site_url'],
      :authorize_url => settings['auth_url'],
      :token_url => settings['token_url'])
  end

  def settings
    @settings ||= Setting.plugin_redmine_omniauth_client
  end

############################

  def group_mapping
    @groups = GroupMapping.all
  end

  def edit
    #mapping = GroupMapping.find_by(variable: params[:variable])
    begin
      mapping = GroupMapping.find_by(variable: params[:variable], :group_id => params[:group_id])
      if mapping == nil
        new_mapping = GroupMapping.new({:variable => params[:variable], :group_id => params[:group_id]})
        new_mapping.save
      else
        mapping.update({:variable => params[:variable], :group_id => params[:group_id]})
    #  mapping.destroy
      end
      flash[:notice] = (t 'notices.saved')
    rescue
      flash[:warning] = (t 'notices.notSaved')
    end
    redirect_to :action => 'group_mapping'
  end

  def delete
    unless GroupMapping.find_by(variable: params[:variable], :group_id => params[:group_id]) == nil
      mapping = GroupMapping.find_by(variable: params[:variable], :group_id => params[:group_id])
      mapping.destroy
    end
    redirect_to :action => 'group_mapping'
  end
end
